require "rspec/datadog/version"

IS_RSPEC3 = Gem::Version.new(RSpec::Core::Version::STRING) >= Gem::Version.new('3')

require 'rspec/core/formatters'
require 'rspec/legacy_formatters' if IS_RSPEC3
require 'rspec/core'

RSpec::Support.require_rspec_core 'formatters/json_formatter' if IS_RSPEC3
require 'active_support'

require 'dogapi'

class DatadogFormatter < RSpec::Core::Formatters::JsonFormatter
  BASE_TAGS = [] # Override this to add custom base tags - name of the system under test, for example

  KEYS_TO_EMIT = [:full_description, :status, :file_path, :line_number].freeze

  if IS_RSPEC3
    RSpec::Core::Formatters.register :self, :message, :dump_summary, :dump_profile, :stop, :close
  end

  def initialize(output)
    # Initialize Dogapi client
    @dog = Dogapi::Client.new(ENV['DATADOG_API_KEY'], ENV['DATADOG_APPLICATION_KEY'])

    super
  end

  def close(_notification=nil)
    # Set now to a value shared by all events for this test run
    now = Time.now.to_i

    datadog_events = @output_hash[:examples].map do |example|
      tags = hash_to_tags(
        example.slice(*KEYS_TO_EMIT)
      )

      Dogapi::Event.new("#{example[:full_description]}: #{example[:status]}",
        :msg_title => "RSpec example",
        :tags => BASE_TAGS + tags,
        :date_happened => now
      )
    end

    datadog_events.each do |event|
      emit_event(event)
    end
  end

  private
  def hash_to_tags(example_hash)
    example_hash.map do |k,v|
      "#{k}:#{v}"
    end
  end

  def emit_event(event)
    if ENV['DEBUG']
      puts JSON.pretty_generate(event.to_hash)
    else
      @dog.emit_event(event)
    end
  end
end
